Package: tram
Title: Transformation Models
Version: 1.4-4
Authors@R: c(person("Torsten", "Hothorn", role = c("aut", "cre"),
                    email = "Torsten.Hothorn@R-project.org", comment = c(ORCID = "0000-0001-8301-0471")),
             person("Luisa", "Barbanti", role = "ctb",
	comment = c(ORCID = "0000-0001-5352-5802")),
             person("Sandra", "Siegfried", role = "aut",
	comment = c(ORCID = "0000-0002-7312-1001")),
             person("Lucas", "Kook", role = c("aut"),
                    email = "lucasheinrich.kook@gmail.com",
                    comment = c(ORCID = "0000-0002-7546-7356")),
             person("Susanne", "Dandl", role = c("ctb"),
                    comment = c(ORCID = "0000-0003-4324-4163")),
             person("Brian", "Ripley", role = "ctb"),
             person("Bill", "Venables", role = "ctb"),
             person(c("Douglas", "M."), "Bates", role = "ctb"),
             person("Nadja", "Klein", role = "ctb"))
Description: Formula-based user-interfaces to specific transformation models
  implemented in package 'mlt' (<DOI:10.32614/CRAN.package.mlt>, <DOI:10.32614/CRAN.package.mlt.docreg>). 
  Available models include Cox models, some parametric 
  survival models (Weibull, etc.), models for ordered categorical variables,
  normal and non-normal (Box-Cox type) linear models, and continuous outcome logistic regression
  (Lohse et al., 2017, <DOI:10.12688/f1000research.12934.1>). The underlying theory
  is described in Hothorn et al. (2018) <DOI:10.1111/sjos.12291>. An extension to
  transformation models for clustered data is provided (Barbanti and Hothorn, 2022,
  <DOI:10.1093/biostatistics/kxac048>) and a tutorial explains applications in survival analysis
  (Siegfried et al., 2025, <DOI:10.48550/arXiv.2402.06428>). Multivariate conditional transformation models 
  (Klein et al, 2022, <DOI:10.1111/sjos.12501>) and shift-scale transformation models (Siegfried et al, 2023,
  <DOI:10.1080/00031305.2023.2203177>) can be fitted as well. The package contains an implementation of 
  a doubly robust score test, described in  Kook et al. (2024, <DOI:10.1080/01621459.2024.2395588>).
Depends: R (>= 4.5.0), mlt (>= 1.8-0), mvtnorm (>= 1.3-7)
Imports: Formula, multcomp, variables (>= 1.0-4), basefun (>= 1.1-2),
        sandwich, stats, survival, graphics, Matrix, methods
Suggests: MASS, TH.data, trtf (>= 0.3-3), mlbench, knitr, quantreg,
        colorspace, ATR, lme4, reformulas, merDeriv, SparseGrid,
        alabama, numDeriv, gridExtra, lattice, latticeExtra, HSAUR3,
        ordinalCont, coxme, mlt.docreg, ordinal, coin, asht, gamlss,
        randomForestSRC, tramME, glmmTMB, geepack, ranger, eha,
        flexsurv, frailtyEM, frailtypack, gamlss.cens, icenReg, mpr,
        rms, rstpm2, timereg, Stat2Data, cotram, latex2exp, tramvs,
        AER, KONPsurv, gamlss.data, xtable, english, bibtex, expm,
        mgcv, pracma
VignetteBuilder: knitr
URL: https://codeberg.org/thothorn/tram
Encoding: UTF-8
License: GPL-2
NeedsCompilation: no
Packaged: 2026-07-16 09:19:14 UTC; hothorn
Author: Torsten Hothorn [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8301-0471>),
  Luisa Barbanti [ctb] (ORCID: <https://orcid.org/0000-0001-5352-5802>),
  Sandra Siegfried [aut] (ORCID: <https://orcid.org/0000-0002-7312-1001>),
  Lucas Kook [aut] (ORCID: <https://orcid.org/0000-0002-7546-7356>),
  Susanne Dandl [ctb] (ORCID: <https://orcid.org/0000-0003-4324-4163>),
  Brian Ripley [ctb],
  Bill Venables [ctb],
  Douglas M. Bates [ctb],
  Nadja Klein [ctb]
Maintainer: Torsten Hothorn <Torsten.Hothorn@R-project.org>
Repository: CRAN
Date/Publication: 2026-07-16 12:50:02 UTC
Built: R 4.6.0; ; 2026-07-16 14:36:57 UTC; unix
