
"Multivariate Conditional Transformation Models"

The two files "sim2d.R" and "sim5d.R" contain simulation code for
reproducing some of the empirical results presented in "Multivariate
Conditional Transformation Models" Alternative implementations with BayesX
and VGAM for the bivariate simulation are included as well, but are
commented out.

"A Transformation Perspective on Marginal and Conditional Models"

mtram_sim.R contains R code for reproducing empirical results presented in
Section 4.
