
### Simulation code for empirical results in
###
###	On Nonparanormal Likelihoods
###	by Torsten Hothorn
###

(1) Adjust number of cores in setup.R

(2) Run R --vanilla < TDA.R
    to reproduce the tables in Section 5.1.

(3) Run ``make -f simMakefile all''
    to reproduce all tables and figures in Section 5.2.

