
### Simulation code for empirical results in
###
###	On Nonparanormal Likelihoods
###	by Torsten Hothorn
###

(0) Choose between bivariate (2d) and multivariate (Jd) setup

(1) Adjust number of cores in setup.R

(2) Run ``make -f simMakefile all''
    to reproduce all tables and figures in Section 5.2.

