
## Reprocibility Material for
##
##      A Transformation Perspective on Marginal and Conditional Models
##      by Luisa Barbanti & Torsten Hothorn, UZH
##      doi:10.1093/biostatistics/kxac048.
##

library("tram")

x <- vignette("mtram", package = "tram")
source(file.path(x$Dir, "doc", x$R), echo = TRUE)
