
## Reprocibility Material for
##
##	Nonparanormal Adjusted Marginal Inference
##	by Susanne Dandl & Torsten Hothorn, UZH
##

library("tram")

x <- vignette("NAMI", package = "tram")
source(file.path(x$Dir, "doc", x$R), echo = TRUE)
